/*
    futebus - FUse TExt BUS - fuse based IPC bus for text lines
    Copyright (C) 2023  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://www.repo.hu/projects/futebus
    VCS: svn://svn.repo.hu/futebus/trunk
    Contact: see doc/contact.html
*/

#include <stdlib.h>
#include "futebus.h"

/* Allocate and return a new, unreferenced message for chan. */
message_t *message_alloc(channel_t *chan, char *body, int len)
{
	message_t *msg = malloc(sizeof(message_t) + len);

	if (msg == NULL)
		return NULL;

	msg->addr = 0;
	msg->refco = 0;
	mutex_init(&msg->refco_lock);
	msg->type = MT_MSG;
	msg->chan = chan;
	memset(&msg->link, 0, sizeof(msg->link));
	msg->len = len;
	if (len > 0)
		memcpy(msg->msg, body, len);
	gdl_append(&chan->messages, msg, link);

	return msg;
}

void message_free(message_t *msg)
{
	gdl_remove(&msg->chan->messages, msg, link);
	mutex_uninit(&msg->refco_lock);
	free(msg);
}

void message_unref(message_t *msg)
{
	mutex_lock(&msg->refco_lock);
	if (msg->refco > 0)
		msg->refco--;
	else
		logprintf("ERROR: msg->refco underflow!\n");
	if (msg->refco == 0)
		message_free(msg);
	else
		mutex_unlock(&msg->refco_lock);
}

